<?php
###############################################################
#         Simple Desk Project - www.simpledesk.net            #
###############################################################
#       An advanced help desk modifcation built on SMF        #
###############################################################
#                                                             #
#         * Copyright 2010 - SimpleDesk.net                   #
#                                                             #
#   This file and its contents are subject to the license     #
#   included with this distribution, license.txt, which       #
#   states that this software is New BSD Licensed.            #
#   Any questions, please contact SimpleDesk.net              #
#                                                             #
###############################################################
# SimpleDesk Version: 1.0 Felidae                             #
# File Info: SimpleDesk-MiscActions.php / 1.0 Felidae         #
###############################################################

/**
 *	This file handles miscellaneous actions that aren't really tied to anything, that are mostly
 *	self-contained and aren't big enough to warrant their own file.
 *
 *	@package source
 *	@since 1.0
 */

if (!defined('SMF'))
	die('Hacking attempt...');

/**
 *	Marks a ticket unread.
 *
 *	There are no permission checks made; other than that the user is who they claim to be. If a ticket is marked unread
 *	but they can't see it anyway, the consequence is that the database gets lighter.
 *
 *	Invoked through ?action=helpdesk;sa=unreadticket;ticket=x;sessvar=sessid before redirecting back to the main helpdesk page.
 *
 *	@since 1.0
*/
function shd_ticket_unread()
{
	global $smcFunc, $user_info, $context;

	is_not_guest();
	checkSession('get');

	if (!empty($context['ticket_id']))
		$result = shd_db_query('', '
			DELETE FROM {db_prefix}helpdesk_log_read
			WHERE id_ticket = {int:current_ticket}
				AND id_member = {int:user}',
			array(
				'current_ticket' => $context['ticket_id'],
				'user' => $user_info['id'],
			)
		);

	redirectexit('action=helpdesk;sa=main');
}

/**
 *	Marks a ticket resolved or unresolved.
 *
 *	This function identifies whether a given ticket is resolved or not, if not resolved, mark it resolved. If it was resolved
 *	reopen the ticket back to an appropriate status based on last respondent. In both cases, the action is logged in the action log.
 *	It is also unassigned from having a user on either closure or reopen.
 *
 *	Accessed through ?action=helpdesk;sa=resolve;ticket=x;sessvar=sessid before redirecting back to the ticket, or add ;home to the
 *	URL to have it redirect back to the home page.
 *
 *	@since 1.0
*/
function shd_ticket_resolve()
{
	global $smcFunc, $user_info, $context, $sourcedir;

	checkSession('get');

	// First, figure out the state of the ticket - is it resolved or not? Can we even see it?
	if (empty($context['ticket_id']))
		fatal_lang_error('shd_no_ticket', false);

	$context['shd_return_to'] = isset($_REQUEST['home']) ? 'home' : 'ticket';

	$query = shd_db_query('', '
		SELECT id_member_started, id_member_updated, status, num_replies, subject
		FROM {db_prefix}helpdesk_tickets AS hdt
		WHERE {query_see_ticket}
			AND id_ticket = {int:current_ticket}',
		array(
			'current_ticket' => $context['ticket_id'],
		)
	);

	if ($row = $smcFunc['db_fetch_assoc']($query))
	{
		$smcFunc['db_free_result']($query);

		if (!shd_allowed_to('shd_resolve_ticket_any') && (!shd_allowed_to('shd_resolve_ticket_own') || $row['id_member_started'] != $user_info['id']))
			fatal_lang_error('shd_cannot_resolve', false);

		$action = ($row['status'] != TICKET_STATUS_CLOSED) ? 'resolve' : 'unresolve';
		$new = shd_determine_status($action, $row['id_member_started'], $row['id_member_updated'], $row['num_replies']);

		shd_db_query('', '
			UPDATE {db_prefix}helpdesk_tickets
			SET status = {int:status},
				id_member_assigned = {int:unassigned}
			WHERE id_ticket = {int:ticket}',
			array(
				'status' => $new,
				'unassigned' => 0,
				'ticket' => $context['ticket_id'],
			)
		);

		shd_log_action($action,
			array(
				'ticket' => $context['ticket_id'],
				'subject' => $row['subject'],
			)
		);

		// Make sure we recalculate the number of tickets on next page load (here it will affect visibility)
		shd_clear_active_tickets($row['id_member_started']);

		if ($context['shd_return_to'] == 'home')
			redirectexit('action=helpdesk;sa=main');
		else
			redirectexit('action=helpdesk;sa=ticket;ticket=' . $context['ticket_id']);
	}
	else
		fatal_lang_error('shd_no_ticket', false);
}

/**
 *	Marks a ticket private/not private if the user isn't using Javascript.
 *
 *	Assuming the user can see the ticket and has suitable permissions, the privacy flag will be inverted for the ticket and
 *	updated, as well as updating the action log. If they cannot see the ticket, or do not have privacy-change permission, a fatal
 *	error will be generated.
 *
 *	Accessed through ?action=helpdesk;sa=privacychange;ticket=x;sessvar=sessid before directing back to the main helpdesk page.
 *
 *	@since 1.0
*/
function shd_privacy_change_noajax()
{
	global $smcFunc, $user_info, $context, $sourcedir;

	checkSession('get');

	// First, figure out the state of the ticket - is it private or not? Can we even see it?
	if (empty($context['ticket_id']))
		fatal_lang_error('shd_no_ticket', false);

	$query = shd_db_query('', '
		SELECT id_member_started, subject, private, status
		FROM {db_prefix}helpdesk_tickets AS hdt
		WHERE {query_see_ticket}
			AND id_ticket = {int:current_ticket}',
		array(
			'current_ticket' => $context['ticket_id'],
		)
	);

	if ($row = $smcFunc['db_fetch_assoc']($query))
	{
		if (in_array($row['status'], array(TICKET_STATUS_CLOSED, TICKET_STATUS_DELETED)) || !shd_allowed_to('shd_alter_privacy_any') && (!shd_allowed_to('shd_alter_privacy_own') || $row['id_member_started'] != $user_info['id']))
			fatal_lang_error('shd_cannot_change_privacy', false);

		$smcFunc['db_free_result']($query);

		$new = empty($row['private']) ? 1 : 0;
		$action = empty($row['private']) ? 'markprivate' : 'marknotprivate';

		require_once($sourcedir . '/Subs-SimpleDeskPost.php');
		$msgOptions = array();
		$posterOptions = array();
		$ticketOptions = array(
			'id' => $context['ticket_id'],
			'private' => empty($row['private']),
		);

		shd_modify_ticket_post($msgOptions, $ticketOptions, $posterOptions);

		shd_log_action($action,
			array(
				'ticket' => $context['ticket_id'],
				'subject' => $row['subject'],
			)
		);

		// Make sure we recalculate the number of tickets on next page load (here it may affect ticket visibility so we do it)
		shd_clear_active_tickets($row['id_member_started']);

		redirectexit('action=helpdesk;sa=ticket;ticket=' . $context['ticket_id']);
	}
	else
		fatal_lang_error('shd_no_ticket', false);
}

/**
 *	Updates a ticket's urgency up or down one level, if the user isn't using Javascript
 *
 *	Assuming the user can see the ticket and has suitable permissions, the urgency of the ticket will be updated and an
 *	action log entry added. If they cannot see the ticket, or do not have urgency-change permission (including higher
 *	urgency permission if necessary), a fatal error will be generated.
 *
 *	Accessed through ?action=helpdesk;sa=urgencychange;ticket=x;change=(increase|decrease);sessvar=sessid before directing back to the main helpdesk page.
 *
 *	@see shd_can_alter_urgency()
 *
 *	@since 1.0
*/
function shd_urgency_change_noajax()
{
	global $smcFunc, $user_info, $context, $sourcedir;

	checkSession('get');

	// First, figure out the state of the ticket - is it private or not? Can we even see it? Current urgency?
	if (empty($context['ticket_id']))
		fatal_lang_error('shd_no_ticket', false);

	$query = shd_db_query('', '
		SELECT id_member_started, subject, urgency, status
		FROM {db_prefix}helpdesk_tickets AS hdt
		WHERE {query_see_ticket}
			AND id_ticket = {int:current_ticket}',
		array(
			'current_ticket' => $context['ticket_id'],
		)
	);

	if ($row = $smcFunc['db_fetch_assoc']($query))
	{
		$can_urgency = shd_can_alter_urgency($row['urgency'], $row['id_member_started'], ($row['status'] == TICKET_STATUS_CLOSED), ($row['status'] == TICKET_STATUS_DELETED));

		if (empty($_GET['change']) || empty($can_urgency[$_GET['change']]))
			fatal_lang_error('shd_cannot_change_urgency');

		$new_urgency = $row['urgency'] + ($_GET['change'] == 'increase' ? 1 : -1);
		$action = 'urgency_' . $_GET['change'];

		require_once($sourcedir . '/Subs-SimpleDeskPost.php');
		$msgOptions = array();
		$posterOptions = array();
		$ticketOptions = array(
			'id' => $context['ticket_id'],
			'private' => empty($row['private']),
		);

		shd_modify_ticket_post($msgOptions, $ticketOptions, $posterOptions);

		shd_log_action($action,
			array(
				'ticket' => $context['ticket_id'],
				'subject' => $row['subject'],
				'urgency' => $new_urgency,
			)
		);

		redirectexit('action=helpdesk;sa=ticket;ticket=' . $context['ticket_id']);
	}
	else
		fatal_lang_error('shd_no_ticket', false);
}
?>